<?php

namespace App\Controllers;

use \System\View;
use Josantonius\Session\Session;
use \System\Auth;
use \App\Models\Account;
use \App\Config;
class Accounts extends \System\Controller
{

    /**
     * Parameters from the matched route
     * @var array
     */
     protected $route_params = [];
    /**
     * Show the index page
     *
     * @return void
     */
    public function __construct($route_params)
    {
        $this->route_params = $route_params;

        Session::init();
        $user = Auth::user();
        if(!$user)
            return header("Location:" . Config::SITEURL . "auth/login");
    }

    public function indexAction() {

        $data = [
            'accounts' => Account::getAll(),
            'added' => isset($_GET['added'])
        ];
        
        return View::renderTemplate('Accounts/list.html', $data);
    }

    public function editAction() {
        if(!isset($this->route_params['username']) || empty($this->route_params['username']))
            exit;

        $username = $this->route_params['username'];
        $account = Account::find($username);

        if(isset($_POST['quota'])) {
            return $this->update($account);
        }

        if(!$account) exit;
        
        return View::renderTemplate('Accounts/edit.html', compact('account'));
    }

    public function createAction() {

        if(isset($_POST['quota'])) {
            $response = Account::create();
            if($response['success']) {
                return header("Location:" . Config::SITEURL . "accounts/index?return=added");
            } else {
                $error = $response['error'];
                return View::renderTemplate('Accounts/create.html', compact('error'));
            }
        }

        return View::renderTemplate('Accounts/create.html');
    }

    
    private function update($account) {
        $response = Account::save($account);
        if($response['success']) {
            return header("Location:" . Config::SITEURL . "accounts/index?return=updated");
        } else {
            $error = $response['error'];
            return View::renderTemplate('Accounts/edit.html', compact('account', 'error'));
        }
    }

    public function deleteAction() {

        if(!isset($this->route_params['username']) || empty($this->route_params['username']))
            exit;

        $username = $this->route_params['username'];
        $account = Account::delete($username);

        return header("Location:" . Config::SITEURL . "accounts/index?return=deleted");
    }
}
