<?php

namespace App\Models;

use PDO;

/**
 * Example user model
 *
 */
class Account extends \System\Model
{

    /**
     * Get all the users as an associative array
     *
     * @return array
     */
     public static function getAll()
     {
         $stmt = static::getDB()->query('SELECT * FROM mailbox');
         $result = $stmt->fetchAll(PDO::FETCH_ASSOC);
         foreach($result as $key => $account) {
            $result[$key]['created'] = date('d/m/Y h:i:s', strtotime($account['created']));
         }
         return $result;
     }
    public static function find( $local_part ) {

        $query = "SELECT * FROM mailbox WHERE local_part = :local_part";
        $result = static::getDB()->prepare($query);

        $result->bindValue(':local_part', $local_part);
        $result->execute();
        if ($result->rowCount() == 0) return false;
        else return $result->fetch(PDO::FETCH_ASSOC);

    }
    
    public static function create() {
        $local_part =  $_POST['username'];
        $username = $local_part . '@'. \App\Config::DOMAIN;
        
        if(!filter_var($username, FILTER_VALIDATE_EMAIL)) 
            return [ 'success' => false, 'error' => 'Invalid username' ];

        // check if email has more than a special chart in sequence
        $re = '/[_.-]{2,}/m';
        if(preg_match($re, $local_part, $matches))
            return [ 'success' => false, 'error' => 'Username can\'t have multiple special character in sequence.' ];

        // check if string has any invalid special char
        $re = '/[a-zA-Z0-9_.-]+/m';
        preg_match_all($re, $local_part, $matches, PREG_SET_ORDER, 0);
        if(count($matches) > 1 || $matches[0][0] != $local_part) 
            return [ 'success' => false, 'error' => 'Username can\'t contain any special characters other than - . _' ];
        

        $len_local_part = strlen($local_part);
        if($len_local_part > 50 || $len_local_part < 3) 
            return [ 'success' => false, 'error' => 'Username must be greater than 3 and less than 50 characters.' ];

        //If the first character is: "-", "_", "." = Invalid, incorrect
        //If the last character is: "-", "_", "." = Invalid, incorrect
        if($local_part[0] == '_' || $local_part[0] == '.' || $local_part[0] == '-'
        || substr($local_part, -1) == '_' || substr($local_part, -1) == '.' || substr($local_part, -1) == '-'
        )
            return [ 'success' => false, 'error' => 'Username can\'t start or end with a special character.' ];
        
        if(!preg_match('/^[a-z0-9 ]+$/i', strtolower($_POST['name']))) {
            return [ 'success' => false, 'error' => 'Special characters aren\'t allowed for name.' ];
        }


        if( $_POST['quota'] < 0 ) 
            return [ 'success' => false, 'error' => 'Invalid quota' ];

        $args = [
            'username' => $username,
            'name' => $_POST['name'],
            'maildir' => $username .'/',
            'quota' =>  $_POST['quota'],
            'local_part' => $local_part,
            'domain' => \App\Config::DOMAIN,
            'active' => 1,
            'phone' => '',
            'email_other' => '',
            'token' => '',
            'token_validity' => date('Y-m-d H:i', strtotime("+365 days")),
            'password_expiry' => date('Y-m-d H:i', strtotime("+365 days"))
        ];

        if(isset($_POST['password'], $_POST['password2']) AND !empty($_POST['password'])) {
            $password_len = strlen($_POST['password']);
            if($password_len < 6 || $password_len > 18) 
                return [
                    'success' => false,
                    'error' => 'Password must be between 6 and 18 characters.'
                ];

            if($_POST['password'] != $_POST['password2']) {
                return [
                    'success' => false,
                    'error' => 'Password and password repeat doesn\'t match'
                ];
            }

            $args['password'] = _pacrypt_dovecot($_POST['password']);
        } else {
            return [
                'success' => false,
                'error' => 'Password field is required'
            ];
        }


        $args = static::preSave(true, $args);
        static::db_insert('mailbox', $args);

        return  ['success' => 1];
    }

    public static function save($account) {
        $args = [
            'quota' => $_POST['quota'],
            'name' => $_POST['name'],
            'active' => (int) isset($_POST['active']) ? $_POST['active'] : 0
        ];

        if( $args['quota'] < 0 ) 
            return [ 'success' => false, 'error' => 'Invalid quota' ];

        if(isset($_POST['password'], $_POST['password2']) AND !empty($_POST['password'])) {
            if($_POST['password'] != $_POST['password2']) {
                return [
                    'success' => false,
                    'error' => 'Password and password repeat doesn\'t match'
                ];
            }

            $args['password'] = _pacrypt_dovecot($_POST['password']);
        }
        $args = static::preSave(false, $args);
        static::db_update('mailbox', 'local_part', $account['local_part'], $args );

        return [
            'success' => true
        ];
    }
    
    protected static function preSave($is_new, $args)
    {
        if (isset($args['quota']) && $args['quota'] != -1 && is_numeric($args['quota'])) {
            $multiplier = \App\Config::MULTIPLIER;
            if ($multiplier == 0 || !is_numeric($multiplier)) { // or empty string, or null, or false...
                $multiplier = 1;
            }
            $args['quota'] = $args['quota'] * $multiplier; # convert quota from MB to bytes
        }

        // Avoid trying to store '' in an integer field
        if ($args['quota'] === '') {
            $args['quota'] = 0;
        }

        // create alias
        if($is_new) {
            $domain = explode('@', $args['username']);
            $ah = [
                'address' => $args['username'] ,
                'goto' => $args['username'],
                'domain' => $domain[1],
                'created' => date('Y-m-d H:i'),
                'modified' => date('Y-m-d H:i'),
                'active' => 1
            ];

            static::db_insert('alias', $ah);
        }

        if (!empty($args['password'])) {
            // provide some default value to keep MySQL etc happy.
            $args['password_expiry'] = date('Y-m-d H:i', strtotime("+365 days"));
        }

        return $args;
    }


    public static function delete($local_part) {
        $query = "DELETE FROM mailbox WHERE local_part = :local_part";
        $result = static::getDB()->prepare($query);

        $result->bindValue(':local_part', $local_part);
        return $result->execute();

        $address = $local_part . "@". \App\Config::DOMAIN;
        $query = "DELETE FROM alias WHERE address = :address";
        $result = static::getDB()->prepare($query);

        $result->bindValue(':address', $address);
        return $result->execute();
    }

}
