<?php

namespace System;

use PDO;
use App\Config;

/**
 * Base model
 *
 */
abstract class Model
{

    public static $db = null; 

    /**
     * Get the PDO database connection
     *
     * @return mixed
     */
    public static function getDB()
    {

        if (static::$db === null) {
            $dsn = 'mysql:host=' . Config::DB_HOST . ';dbname=' . Config::DB_NAME . ';charset=utf8';
            static::$db  = new PDO($dsn, Config::DB_USER, Config::DB_PASSWORD);

            // Throw an Exception when an error occurs
            static::$db ->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
        }

        return static::$db;
    }

    public static function db_insert(string $table, array $values, array $timestamp = array('created', 'modified'), bool $throw_exceptions = false) {
        $table = "`" . $table . "`";

        foreach ($timestamp as $key) {
            $values[$key] = "now()";
        }

        $value_string = '';
        $comma = '';
        $prepared_statment_values = $values;

        foreach ($values as $field => $value) {
            if (in_array($field, $timestamp)) {
                $value_string .= $comma . $value; // see above.
                unset($prepared_statment_values[$field]);
            } else {
                $value_string .= $comma . ":{$field}";
            }
            $comma = ',';
        }

        return static::db_execute(
            "INSERT INTO $table (" . implode(",", array_keys($values)) .") VALUES ($value_string)",
            $prepared_statment_values,
            $throw_exceptions);
    }

    /**
    * db_update
    * Action: Updates a specified table
    * Call: db_update (string table, string where_col, string where_value, array values [, array timestamp])
    * @param string $table - table name
    * @param string $where_col - column of WHERE condition
    * @param string $where_value - value of WHERE condition
    * @param array $values - key/value map of data to insert into the table.
    * @param array $timestamp (optional) - array of fields to set to now() - default: array('modified')
    * @return int - number of updated rows
    */
    public static function db_update(string $table, string $where_col, string $where_value, array $values, array $timestamp = array('modified'), bool $throw_exceptions = false):int
    {
        $table = "`" . $table . "`";
    
        $pvalues = array();
    
        $set = array();
    
        foreach ($timestamp as $k) {
            if (!isset($values[$k])) {
                $values[$k] = 'x'; // timestamp field not in the values list, add it in so we set it to now() see #469
            }
        }
    
        foreach ($values as $key => $value) {
            if (in_array($key, $timestamp)) {
                $set[] = " $key = now() ";
                continue;
            }
    
            $set[] = " $key = :$key ";
            $pvalues[$key] = $value;
        }
    
        $pvalues['where'] = $where_value;
    
    
        $sql="UPDATE $table SET " . implode(",", $set) . " WHERE $where_col = :where";
        return static::db_execute($sql, $pvalues, $throw_exceptions);
    }

    
    /**
    * @param string $sql e.g. UPDATE foo SET bar = :baz
    * @param array $values - parameters for the prepared statement e.g. ['baz' => 1234]
    * @param bool $throw_exceptions
    * @return int number of rows affected by the query
    */
    private static function db_execute($sql, array $values = [], $throw_exceptions = false)
    {
        $stmt = static::getDB()->prepare($sql);
        $stmt->execute($values);

        return $stmt->rowCount();
    }
    
}
